package Sledge::Plugin::Validator::email_super_loose;
use strict;
use vars qw($VERSION);
$VERSION = '0.02';

sub load {
	my $self = shift;
	$self->set_function(
		EMAIL_SUPER_LOOSE    => \&is_EMAIL_SUPER_LOOSE,
	);
}

sub is_EMAIL_SUPER_LOOSE {
	local $_ = shift;

	my @email = split/\@/;
	if (
		/^[\x21-\x7E]+$/    and #  ASCII
		scalar(@email) == 2 and # @ 1
		/\@.+[\.].+$/       and # @ θ . ϤҤȤİʾ
		/\@[a-zA-Z0-9]/         # @ ľϱѿ
	) {
		return 1;
	}
	else {
		return 0;
	}
}

1;

=head1 NAME

Sledge::Plugin::Validator::email_super_loose - Ķñʥ᡼륢ɥ쥹Υå

=head1 SYNOPSIS

  $self->valid->check(
      email => [qw(EMAIL_SUPER_LOOSE)]
  );

=head1 DESCRIPTION

᡼륢ɥ쥹δñʽ񼰥åԤޤ

=head1 CHECK FUNCTION

=over 4

=item EMAIL_SUPER_LOOSE

 ASCII@ 1@ θ . ϤҤȤİʾ塢@ ľϱѿ
ȤåΤߤԤޤ

ºݡRFC򤸤ʤɥ쥹󤵤¸ߤƤΤ¡
(ä˷ä)
᡼륢ɥ쥹ߤС᡼ƤߤΤ
ɤ櫓ǡ

ξ硢åƻȤ᡼륢ɥ쥹Ϥ
⤤ʡȤäơޤ

=back

=head1 AUTHOR

KIMURA, takefumi E<lt>takefumi@takefumi.comE<gt>

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=head1 SEE ALSO

L<Sledge::Plugin::Validator>

=cut

